@@ -87,11 +87,18 @@ int phork()
     return pid;
 }
 
+
+static int list_pipe = 0, nowait, list_pipe_pid, pline_level = 0;
+static int list_pipe_child = 0, list_pipe_job;
+static char list_pipe_text[JOBTEXTSIZE];
+
 /* execute a current shell command */
 
 int execcursh(cmd)		/**/
 Cmd cmd;
 {
+    if (!list_pipe)
+	deletejob(jobtab + thisjob);
     runlist(cmd->u.list);
     cmd->u.list = NULL;
     return lastval;
@@ -199,11 +206,11 @@ int dash;
     if (cn && (cn->flags & DISABLED))
 	cn = NULL;
     if ((z = zgetenv("ARGV0"))) {
-	setdata(firstnode(args), (vptr) ztrdup(z));
+	setdata(firstnode(args), (void_ptr) ztrdup(z));
 	delenv(z - 6);
     } else if (dash) {
 	sprintf(buf2, "-%s", arg0);
-	setdata(firstnode(args), (vptr) ztrdup(buf2));
+	setdata(firstnode(args), (void_ptr) ztrdup(buf2));
     }
     argv = makecline(args);
     unblockchld();
@@ -478,24 +485,19 @@ int last1;
     }
 }
 
-static int in_pipe;
-
 int execpline(l, how, last1)	/**/
 Sublist l;
 int how;
 int last1;
 {
     int ipipe[2], opipe[2];
+    int pj = thisjob, nj;
 
     if (!l)
 	return 0;
     ipipe[0] = ipipe[1] = opipe[0] = opipe[1] = 0;
-    if (in_pipe) {
-	execpline2(l->left, how, opipe[0], ipipe[1], last1);
-	return lastval;		/* if we're in a pipe don't change lastval; it'll be */
-    }				/* done by the first instance of execpline */
     blockchld();
-    if ((thisjob = getfreejob()) == -1)
+    if ((thisjob = nj = getfreejob()) == -1)
 	return 1;
     initjob();
     if (how == TIMED) {
@@ -513,7 +515,12 @@ int last1;
 	coprocin = ipipe[0];
 	coprocout = opipe[1];
     }
+    if (!pline_level++) {
+	list_pipe_job = nj;
+	nowait = 0;
+    }
     execpline2(l->left, how, opipe[0], ipipe[1], last1);
+    pline_level--;
     if (how == ASYNC) {
 	if (l->flags & PFLAG_COPROC)
 	    close(ipipe[1]);
@@ -521,10 +528,115 @@ int last1;
 	unblockchld();
 	return 1;
     } else {
-	waitjobs();
+	Job jn = jobtab + nj;
+
+	if (nj == list_pipe_job && list_pipe_child)
+	    _exit(0);
+
+	thisjob = nj;
+
+	if (list_pipe)
+	    jn->stat |= STAT_NOPRINT;
+
+	if (nowait) {
+	    if(!pline_level) {
+		struct process *pn, *qn;
+
+		curjob = nj;
+		addproc(list_pipe_pid, list_pipe_text);
+
+		for (pn = jobtab[jn->other].procs; pn; pn = pn->next)
+		    if (WIFSTOPPED(pn->statusp))
+			break;
+
+		if (pn) {
+		    for (qn = jn->procs; qn->next; qn = qn->next);
+		    qn->statusp = pn->statusp;
+		}
+
+		jn->stat &= ~(STAT_DONE | STAT_NOPRINT);
+		jn->stat |= STAT_STOPPED | STAT_CHANGED;
+		printjob(jn, !!isset(LONGLISTJOBS));
+	    }
+	    else
+		deletejob(jn);
+	}
+
+	for (; !nowait;) {
+	    if (list_pipe_child) {
+		jn->stat |= STAT_NOPRINT;
+		makerunning(jn);
+	    }
+	    waitjobs();
+
+	    if (list_pipe_child &&
+		jn->stat & STAT_DONE &&
+		lastval2 & 0200)
+		killpg(mypgrp, lastval2 & ~0200);
+	    if ((list_pipe || exiting) && !list_pipe_child &&
+		jn->stat & STAT_STOPPED) {
+		int synch[2], pid;
+
+		pipe(synch);
+
+		if ((pid = fork()) == -1) {
+		    trashzle();
+		    close(synch[0]);
+		    close(synch[1]);
+		    putc('\n', stderr);
+		    fprintf(stderr, "zsh: job can't be suspended\n");
+		    fflush(stderr);
+		    makerunning(jn);
+		    killjb(jn, SIGCONT);
+		    thisjob = nj;
+		}
+		else if (pid) {
+		    char dummy;
+
+		    list_pipe_pid = pid;
+		    nowait = errflag = 1;
+		    breaks = loops;
+		    close(synch[1]);
+		    read(synch[0], &dummy, 1);
+		    close(synch[0]);
+		    jobtab[list_pipe_job].other = nj;
+		    jobtab[list_pipe_job].stat |= STAT_SUPERJOB;
+		    jn->stat |= STAT_SUBJOB | STAT_NOPRINT;
+		    jn->other = pid;
+		    killpg(jobtab[list_pipe_job].gleader, SIGSTOP);
+		    break;
+		}
+		else {
+		    close(synch[0]);
+		    entersubsh2(1, 0);
+		    setpgrp(0L, mypgrp = jobtab[list_pipe_job].gleader);
+		    close(synch[1]);
+		    kill(getpid(), SIGSTOP);
+		    list_pipe = 0;
+		    list_pipe_child = 1;
+		    break;
+		}
+	    }
+	    else if (subsh && jn->stat & STAT_STOPPED)
+		thisjob = nj;
+	    else
+		break;
+	}
 	unblockchld();
+
+	if (list_pipe && (lastval & 0200) && pj >= 0 &&
+	    (!(jn->stat & STAT_INUSE) || (jn->stat & STAT_DONE))) {
+	    jn = jobtab + pj;
+	    jn->stat |= STAT_NOPRINT;
+	    killjb(jobtab + pj, lastval & ~0200);
+	}
+	if (list_pipe_child)
+	    deletejob(jn);
+	thisjob = pj;
+
 	if (l->flags & PFLAG_NOT)
 	    lastval = !lastval;
+
 	return !lastval;
     }
 }
@@ -545,14 +657,14 @@ int last1;
     if (!pline)
 	return;
     lineno = pline->left->lineno;
+    if (pline_level == 1)
+	strcpy(list_pipe_text, getjobtext((void_ptr) pline->left));
     if (pline->type == END) {
-	in_pipe = 0;
 	execcmd(pline->left, input, output, how == ASYNC, last1);
 	pline->left = NULL;
     } else {
-	int old_pipe = in_pipe;
+	int old_list_pipe = list_pipe;
 
-	in_pipe = 0;
 	mpipe(pipes);
 	if (pline->left->type >= CURSH && how == SYNC) {
 
@@ -576,7 +688,7 @@ int last1;
 		close(synch[1]);
 		zerr("fork failed: %e", NULL, errno);
 	    } else {
-		char dummy, *text = getjobtext((vptr) pline->left);
+		char dummy, *text = getjobtext((void_ptr) pline->left);
 
 		addproc(pid, text);
 		close(synch[1]);
@@ -587,14 +699,14 @@ int last1;
 	/* otherwise just do the pipeline normally. */
 	    execcmd(pline->left, input, pipes[1], how == ASYNC, 0);
 	}
-	in_pipe = old_pipe;
 	pline->left = NULL;
 	close(pipes[1]);
 	if (pline->right) {
 	/* if another execpline() is invoked because the command is a list it
 	   must know that we're already in a pipeline */
-	    in_pipe = 1;
+	    list_pipe = 1;
 	    execpline2(pline->right, how, pipes[0], output, last1);
+	    list_pipe = old_list_pipe;
 	    close(pipes[0]);
 	}
     }
@@ -871,7 +983,7 @@ int last1;
 	    pushnode(args, dupstring("fg"));
     }
     if (jobbing) {		/* get the text associated with this command */
-	text = getjobtext((vptr) cmd);
+	text = getjobtext((void_ptr) cmd);
     } else
 	text = NULL;
     if (full(args) && !(cmd->flags & CFLAG_COMMAND) && type == SIMPLE) {
@@ -898,6 +1010,12 @@ int last1;
     prefork(args, (((type == CCASE) ? 010 : 0)
 		   | (assign ? 02 : isset(MAGICEQUALSUBST))));
 
+    zsfree(underscore);
+    if (full(args) && (underscore = ztrdup((char *) getdata(lastnode(args)))))
+	untokenize(underscore); 
+    else
+  	underscore = ztrdup("");
+
  /* warn about "rm *" */
     if (unset(RMSTARSILENT) && interact && isset(SHINSTDIN) &&
 	type == SIMPLE && full(args) && nextnode(firstnode(args)) &&
@@ -947,7 +1065,7 @@ int last1;
 	if (type == SIMPLE && !cn1 && isset(AUTOCD) && isset(SHINSTDIN) &&
 	    full(args) && empty(cmd->redir) &&
 	    !nextnode(firstnode(args)) && (s = cancd(peekfirst(args)))) {
-	    peekfirst(args) = (vptr) s;
+	    peekfirst(args) = (void_ptr) s;
 	    pushnode(args, dupstring("cd"));
 	    cn1 = (Cmdnam) gethnode("cd", cmdnamtab);
 	}
@@ -1117,7 +1235,7 @@ int last1;
     }
     if (unset(NOEXEC))
 	if (type >= CURSH) {
-	    static int (*func[]) DCLPROTO((Cmd)) =
+	    static int (*func[]) _((Cmd)) =
 	    {
 		execcursh, exectime, execfuncdef, execfor, execwhile,
 		execrepeat, execif, execcase, execselect, execcond};
@@ -1129,6 +1247,25 @@ int last1;
 		lastval = 1;
 		return;
 	    }
+	    
+	    if (!forked) {
+		Lknode n;
+		Param pm;
+		char *s;
+
+		oldlocallist = locallist;
+		locallist = newlist();
+		locallevel++;
+		
+		for (n = firstnode(cmd->vars); n; incnode(n)) {
+		    addnode(locallist,
+			    dupstring(s = ((struct varasg *) getdata(n))->name));
+		    
+		    pm = createparam(ztrdup(s), 0);
+		    pm->level = locallevel;
+		}
+	    }
+	    
 	    if (cmd->vars) {
 		addvars(cmd->vars, 0);
 		if (errflag) {
@@ -1157,6 +1294,21 @@ int last1;
 		} else
 		    clearerr(stdout);
 	    }
+
+	    if (!forked) {
+		char *s;
+
+		locallevel--;
+
+		while ((s = (char *)ugetnode(locallist))) {
+		    Param pm = (Param) gethnode(s, paramtab);
+
+		    if (pm && pm->level > locallevel)
+			unsetparam(s);
+		}
+		locallist = oldlocallist;
+	    }
+		
 	} else {
 	    if (cmd->flags & CFLAG_EXEC)
 		setiparam("SHLVL", --shlvl);
@@ -1170,8 +1322,10 @@ int last1;
 	    if (type == SIMPLE) {
 		closem();
 		execute(cmd->flags & CFLAG_DASH);
-	    } else		/* ( ... ) */
+	    } else {		/* ( ... ) */
+		list_pipe = 0;
 		execlist(cmd->u.list);
+	    }
 	}
   err:
     if (forked)
@@ -1193,8 +1347,9 @@ int *save;
     errno = old_errno;
 }
 
-void entersubsh(bkg)		/**/
+void entersubsh2(bkg, cl)		/**/
 int bkg;
+int cl;
 {
     if (!jobbing) {
 	if (bkg) {
@@ -1210,9 +1365,14 @@ int bkg;
 		}
 	    }
 	}
-    } else if (thisjob != -1) {
-	if (!jobtab[thisjob].gleader) {
+    } else if (thisjob != -1 && cl) {
+	if (list_pipe || list_pipe_child)
+	    setpgrp(0L, jobtab[list_pipe_job].gleader);
+	else if (!jobtab[thisjob].gleader) {
 	    jobtab[thisjob].gleader = getpid();
+	    if (list_pipe_job != thisjob &&
+		!jobtab[list_pipe_job].gleader)
+		jobtab[list_pipe_job].gleader = jobtab[thisjob].gleader;
 	    setpgrp(0L, jobtab[thisjob].gleader);
 	    if (!bkg)
 		attachtty(jobtab[thisjob].gleader);
@@ -1239,7 +1399,14 @@ int bkg;
 	sig_default(SIGQUIT);
     opts[MONITOR] = OPT_UNSET;
     opts[USEZLE] = OPT_UNSET;
-    clearjobtab();
+    if (cl)
+	clearjobtab();
+}
+
+void entersubsh(bkg)		/**/
+int bkg;
+{
+    entersubsh2(bkg, 1);
 }
 
 /* close all internal shell fds */
@@ -1586,6 +1753,7 @@ char *cmd;
     subsh = 1;
     close(1);
     entersubsh(0);
+    sig_ignore(SIGTSTP);
     (void)creat(nam, 0666);
     exiting = 1;
     execlist(list);
@@ -1966,6 +2134,8 @@ Cmdnam cn;
 
     if (errflag)
 	return;
+    if (!list_pipe)
+	deletejob(jobtab + thisjob);
     if (!cn->u.list) {
 	char *nam;
 
@@ -2110,6 +2280,19 @@ char *s;
     (s[1] == '/' || !s[1] || (s[1] == '.' && (s[2] == '/' || !s[1])));
     char *t;
 
+    if (haswilds(s)) {
+	Lklist l = newlist();
+	int ne = noerrs;
+
+	noerrs = 1;
+	addnode(l, dupstring(s));
+	postfork(l, 1);
+	if (!errflag && full(l) && !nextnode(firstnode(l)))
+	    s = peekfirst(l);
+	errflag = 0;
+	noerrs = ne;
+    }
+
     if (*s != '/') {
 	char sbuf[MAXPATHLEN], **cp;
 
