@@ -32,31 +32,31 @@ struct asgment {
 #include "zle_vi.pro"
 #include "zle_word.pro"
 
-char *mktemp DCLPROTO((char *));
+char *mktemp _((char *));
 
 #ifndef HAS_STDLIB
-char *malloc DCLPROTO((int));
-char *realloc DCLPROTO((char *, int));
-char *calloc DCLPROTO((int, int));
+char *malloc _((int));
+char *realloc _((char *, int));
+char *calloc _((int, int));
 
 #endif
-char *ttyname DCLPROTO((int));
+char *ttyname _((int));
 
 extern char PC, *BC, *UP;
 extern short ospeed;
-extern int tgetent DCLPROTO((char *bp, char *name));
-extern int tgetnum DCLPROTO((char *id));
-extern int tgetflag DCLPROTO((char *id));
-extern char *tgetstr DCLPROTO((char *id, char **area));
-extern char *tgoto DCLPROTO((char *cm, int destcol, int destline));
-extern int tputs DCLPROTO((char *cp, int affcnt, int (*outc) (int)));
+extern int tgetent _((char *bp, char *name));
+extern int tgetnum _((char *id));
+extern int tgetflag _((char *id));
+extern char *tgetstr _((char *id, char **area));
+extern char *tgoto _((char *cm, int destcol, int destline));
+extern int tputs _((char *cp, int affcnt, int (*outc) (int)));
 
 /* missing prototypes for various C compilers */
 
 #if defined(SOLARIS)
 #include <setjmp.h>
 /* Solaris does not seem to have prototype for these under /usr/include */
-extern char *getdomainname DCLPROTO((char *name, int namelen));
+extern char *getdomainname _((char *name, int namelen));
 extern FILE *fdopen(int, const char *);
 extern int kill(pid_t, int);
 extern int sigaction(int, const struct sigaction *, struct sigaction *);
@@ -69,7 +69,6 @@ extern int gettimeofday(struct timeval *tp, struct timezone *tzp);
 
 #if defined(__convexc__)
 /* ConvexOS does not seem to have prototypes for these under /usr/include */
-extern int bzero(char *b, int length);
 extern int gethostname(char *name, int namelen);
 extern int getrlimit(int resource, struct rlimit *rlp);
 extern int getrusage(int, struct rusage *);
@@ -89,7 +88,7 @@ extern int wait3(int *status, int options, struct rusage *rusage);
 
 /* HP/UX 9 c89 */
 #if defined(__hpux) && defined(_XPG3) && !defined(_POSIX1_1988)
-#define WRITE_ARG_2_T void *
+#define WRITE_ARG_2_T void_ptr
 #else
 #define WRITE_ARG_2_T char *
 #endif
@@ -115,7 +114,7 @@ extern struct passwd *getpwent(void);
 extern int BSDgettimeofday(struct timeval *tp, struct timezone *tzp);
 extern int getrusage(int rwho, struct rusage *rusage);
 extern int lstat(const char *path, struct stat *buf);
-extern int readlink(const char *path, void *buf, size_t bufsiz);
+extern int readlink(const char *path, void_ptr buf, size_t bufsiz);
 
 #endif
 extern int wait3(union wait *statptr, int options, struct rusage *rusage);
@@ -126,7 +125,6 @@ extern int kill(pid_t pid, int sig);
 extern int readlink(const char *pathname, char *buf, int bufsiz);
 
 #endif
-extern void bzero(void *b, int length);
 extern int gethostname(char *name, int namelen);
 extern int ioctl(int fildes, int request,...);
 extern int mknod(const char *pathname, mode_t mode, dev_t dev);
@@ -154,7 +152,6 @@ extern void siglongjmp __((sigjmp_buf _Env, int _Val));
 
 #if defined(__NeXT__)
 #if defined(__STRICT_ANSI__)	/* -ansi does not see these */
-#define bzero(b,len) memset(b,0,len)
 extern FILE *fdopen(int filedes, const char *mode);
 
 #endif
@@ -167,7 +164,6 @@ extern int getppid(void);
 
 #if defined(__osf__) && defined(__alpha)
 /* Digital cc does not need these prototypes, gcc does need them */
-void bzero(char *string, int length);
 int gethostname(char *address, int address_len);
 int ioctl(int d, unsigned long request, char *argp);
 int mknod(const char *pathname, int mode, dev_t device);
@@ -186,7 +182,6 @@ extern int gettimeofday(struct timeval *time_value,
 extern int wait3(union wait *wait_status, int options, struct rusage *rusage);
 extern int gethostname(char *nameptr, int maxlength);
 extern int getdomainname(char *name, int maxlength);
-extern int bzero(char *ptr, int length);
 extern int select(int nfds, fd_set * readfds, fd_set * writefds,
 		  fd_set * exceptfds, struct timeval *timeout);
 
