@@ -127,9 +127,9 @@ const char *zshname;
 {
     int c;
 
-    for (c = 0; c != 43; c++)
+    for (c = 0; c != 45; c++)
 	opts[c] = OPT_UNSET;
-    for (c = 43; c != 128; c++)
+    for (c = 45; c != 128; c++)
 	opts[c] = OPT_INVALID;
     for (c = 'a'; c <= 'z'; c++)
 	opts[c] = opts[c - 'a' + 'A'] = OPT_UNSET;
@@ -309,7 +309,7 @@ void setmoreflags()
 #endif
 }
 
-SPROTO(long get_baudrate, (int speedcode));
+static long get_baudrate _((int speedcode));
 
 static long get_baudrate(speedcode)
 int speedcode;
@@ -473,6 +473,7 @@ const char *zshname;
     hostnam = (char *)zalloc(256);
     underscore = ztrdup("");
     gethostname(hostnam, 256);
+    hostvar = ztrdup(hostnam);
     mypid = getpid();
     cdpath = mkarray(NULL);
     manpath = mkarray(NULL);
