@@ -69,7 +69,7 @@ static struct z_jmp_buf {
     JMP_BUF jbuf;
 } foil_race;
 
-void chldhandler DCLPROTO((struct z_jmp_buf * jump));
+void chldhandler _((struct z_jmp_buf * jump));
 
 void chldsuspend(sig)		/**/
 int sig;
@@ -188,15 +188,15 @@ struct z_jmp_buf *jump;
 	int old_errno = errno;
 
 #ifdef HAS_RUSAGE
-	pid = wait3((vptr) & statusp, WNOHANG | WUNTRACED, &ru);
+	pid = wait3((void_ptr) & statusp, WNOHANG | WUNTRACED, &ru);
 #else
 #ifndef WNOHANG
 	pid = wait(&statusp);
 #else
 #ifdef HAS_WAITPID
-	pid = waitpid(-1, (vptr) & statusp, WNOHANG | WUNTRACED);
+	pid = waitpid(-1, (void_ptr) & statusp, WNOHANG | WUNTRACED);
 #else
-	pid = wait3((vptr) & statusp, WNOHANG | WUNTRACED, NULL);
+	pid = wait3((void_ptr) & statusp, WNOHANG | WUNTRACED, NULL);
 #endif
 #endif
 	chlds = shtms.tms_cstime;
@@ -262,8 +262,12 @@ Job jn;
 
     jn->stat &= ~STAT_STOPPED;
     for (pn = jn->procs; pn; pn = pn->next)
-	if (WIFSTOPPED(pn->statusp))
+	if (WIFSTOPPED(pn->statusp) && 
+	    (!(jn->stat & STAT_SUPERJOB) || pn->next))
 	    pn->statusp = SP_RUNNING;
+
+    if (jn->stat & STAT_SUPERJOB)
+	makerunning(jobtab + jn->other);
 }
 
 /* update status of job, possibly printing it */
@@ -375,6 +379,9 @@ int lng;
     int conted = 0, lineleng = columns, skip = 0, doputnl = 0;
     struct process *pn;
 
+    if (jn->stat & STAT_NOPRINT)
+	return;
+
     if (lng < 0) {
 	conted = 1;
 	lng = 0;
@@ -606,9 +613,15 @@ int execok()
 void waitforpid(pid)		/**/
 long pid;
 {
+    int first = 1;
 /* blockchld() around this loop in case #ifndef WNOHANG */
     blockchld();		/* unblocked in chldsuspend() */
     while (!errflag && (kill(pid, 0) >= 0 || errno != ESRCH)) {
+	if (first)
+	    first = 0;
+	else
+	    kill(pid, SIGCONT);
+
 	chldsuspend(SIGINT);
 	blockchld();
     }
@@ -632,6 +645,30 @@ int sig;
 	       !(jn->stat & STAT_DONE) &&
 	       !(interact && (jn->stat & STAT_STOPPED))) {
 	    chldsuspend(sig);
+	    if (jn->stat & STAT_SUPERJOB) {
+		Job sj = jobtab + jn->other;
+
+		if (sj->stat & STAT_DONE) {
+		    struct process *p;
+		    
+		    for (p = sj->procs; p; p = p->next)
+			if (WIFSIGNALED(p->statusp)) {
+			    killpg(jn->gleader, WTERMSIG(p->statusp));
+			    break;
+			}
+		    if (!p) {
+			jn->stat &= ~STAT_SUPERJOB;
+			kill(sj->other, SIGCONT);
+			deletejob(sj);
+		    }
+		    curjob = jn - jobtab;
+		}
+		else if (sj->stat & STAT_STOPPED) {
+		    jn->stat |= STAT_STOPPED;
+		    curjob = jn - jobtab;
+		    break;
+		}
+	    }
 	    blockchld();
 	}
     } else
@@ -922,6 +959,7 @@ void killrunjobs()
 	return;
     for (t0 = 1; t0 != MAXJOB; t0++)
 	if ((from_sig || t0 != thisjob) && (jobtab[t0].stat & STAT_LOCKED) &&
+	    !(jobtab[t0].stat & STAT_NOPRINT) &&
 	    !(jobtab[t0].stat & STAT_STOPPED)) {
 	    if (killpg(jobtab[t0].gleader, SIGHUP) != -1)
 		killed++;
@@ -938,7 +976,8 @@ void checkjobs()
 
     scanjobs();
     for (t0 = 1; t0 != MAXJOB; t0++)
-	if (t0 != thisjob && jobtab[t0].stat & STAT_LOCKED)
+	if (t0 != thisjob && (jobtab[t0].stat & STAT_LOCKED) &&
+	    !(jobtab[t0].stat & STAT_NOPRINT))
 	    break;
     if (t0 != MAXJOB) {
 	if (jobtab[t0].stat & STAT_STOPPED) {
@@ -962,8 +1001,23 @@ int sig;
     struct process *pn;
     int err = 0;
 
-    if (jobbing)
-	return (killpg(jn->gleader, sig));
+    if (jobbing) {
+	if (jn->stat & STAT_SUPERJOB) {
+	    if (sig == SIGCONT) {
+		for (pn = jobtab[jn->other].procs; pn; pn = pn->next)
+		    kill(pn->pid, sig);
+
+		for (pn = jn->procs; pn->next; pn = pn->next)
+		    err = kill(pn->pid, sig);
+		return err;
+	    }
+
+	    killpg(jobtab[jn->other].gleader, sig);
+	    return killpg(jn->gleader, sig);
+	}
+	else
+	    return (killpg(jn->gleader, sig));
+    }
     for (pn = jn->procs; pn; pn = pn->next)
 	if ((err = kill(pn->pid, sig)) == -1 && errno != ESRCH)
 	    return -1;
