@@ -20,8 +20,8 @@
 
 #include "zsh.h"
 
-vptr(*alloc) DCLPROTO((int));
-vptr(*ncalloc) DCLPROTO((int));
+void_ptr(*alloc) _((int));
+void_ptr(*ncalloc) _((int));
 
 /*
 
@@ -75,8 +75,8 @@ void heapalloc()
     useheap = 1;
 }
 
-static vptr(*lastcalloc) DCLPROTO((int));
-static vptr(*lastncalloc) DCLPROTO((int));
+static void_ptr(*lastcalloc) _((int));
+static void_ptr(*lastncalloc) _((int));
 static int lastuseheap;
 
 /* set default allocation to malloc() */
@@ -191,7 +191,7 @@ void popheap()
 
 /* allocate memory from the current memory pool */
 
-vptr halloc(size)		/**/
+void_ptr halloc(size)		/**/
 int size;
 {
     Heap h, hp;
@@ -216,10 +216,10 @@ int size;
 
 #ifdef USE_ZSH_MALLOC
 	static int called = 0;
-	vptr foo;
+	void_ptr foo;
 
 	if (called)
-	    foo = (vptr) malloc(HEAPFREE);
+	    foo = (void_ptr) malloc(HEAPFREE);
 #endif
 
 	for (hp = NULL, h = heaps; h; hp = h, h = h->next);
@@ -250,22 +250,22 @@ int size;
 
     zigsafe();
 
-    return (vptr) ret;
+    return (void_ptr) ret;
 }
 
 /* allocate memory from the current memory pool and clear it */
 
-vptr hcalloc(size)		/**/
+void_ptr hcalloc(size)		/**/
 int size;
 {
-    vptr ptr;
+    void_ptr ptr;
 
     ptr = halloc(size);
     memset(ptr, 0, size);
     return ptr;
 }
 
-vptr hrealloc(p, old, new)	/**/
+void_ptr hrealloc(p, old, new)	/**/
 char *p;
 int old;
 int new;
@@ -274,21 +274,21 @@ int new;
 
     ptr = (char *)halloc(new);
     memcpy(ptr, p, old);
-    return (vptr) ptr;
+    return (void_ptr) ptr;
 }
 
 /* allocate permanent memory */
 
-vptr zalloc(l)			/**/
+void_ptr zalloc(l)			/**/
 int l;
 {
-    vptr z;
+    void_ptr z;
 
     zigunsafe();
 
     if (!l)
 	l = 1;
-    if (!(z = (vptr) malloc(l))) {
+    if (!(z = (void_ptr) malloc(l))) {
 	zerr("fatal error: out of memory", NULL, 0);
 	exit(1);
     }
@@ -297,10 +297,10 @@ int l;
     return z;
 }
 
-vptr zcalloc(size)		/**/
+void_ptr zcalloc(size)		/**/
 int size;
 {
-    vptr ptr;
+    void_ptr ptr;
 
     ptr = zalloc(size);
     memset(ptr, 0, size);
@@ -365,13 +365,13 @@ const char *s;
 #if !defined(__hpux) && !defined(DGUX)
 #if defined(_BSD) && !defined(SYSV)
 
-extern int brk DCLPROTO((caddr_t));
-extern caddr_t sbrk DCLPROTO((int));
+extern int brk _((caddr_t));
+extern caddr_t sbrk _((int));
 
 #else
 
-extern int brk DCLPROTO((void *));
-extern void *sbrk DCLPROTO((int));
+extern int brk _((void_ptr));
+extern void_ptr sbrk _((int));
 
 #endif
 #endif
@@ -387,8 +387,8 @@ extern void *sbrk DCLPROTO((int));
 #else
 
 #define FREE_RET_T void
-#define FREE_ARG_T void *
-#define MALLOC_RET_T void *
+#define FREE_ARG_T void_ptr
+#define MALLOC_RET_T void_ptr
 #define MALLOC_ARG_T size_t
 
 #endif
@@ -446,16 +446,16 @@ MALLOC_ARG_T size;
 
     if (!m_pgsz) {
 #ifdef __hpux
-	extern long sysconf DCLPROTO((int));
+	extern long sysconf _((int));
 
 	m_pgsz = sysconf(_SC_PAGE_SIZE);
 #else
 #ifdef SYSV
-	extern long sysconf DCLPROTO((int));
+	extern long sysconf _((int));
 
 	m_pgsz = sysconf(_SC_PAGESIZE);
 #else
-	extern int getpagesize DCLPROTO((void));
+	extern int getpagesize _((void));
 
 	m_pgsz = getpagesize();
 #endif
@@ -603,7 +603,7 @@ MALLOC_ARG_T size;
 }
 
 void zfree(p, sz)		/**/
-vptr p;
+void_ptr p;
 int sz;
 {
     struct m_hdr *m = (struct m_hdr *)(((char *)p) - M_ISIZE), *mp, *mt;
@@ -679,7 +679,7 @@ int sz;
 		    m_small[i] = mt->next;
 
 		m = mt;
-		p = (vptr) & m->next;
+		p = (void_ptr) & m->next;
 
 		break;
 	    } else if (sz) {
@@ -974,7 +974,7 @@ int func;
 #else				/* not USE_ZSH_MALLOC */
 
 void zfree(p, sz)		/**/
-vptr p;
+void_ptr p;
 int sz;
 {
     if (p)
