@@ -845,24 +845,13 @@ Cmd c;
 	    nocorrect = 1;
 	else if (tok == DASH)
 	    c->flags = CFLAG_DASH;
-	else
-	    break;
-	yylex();
-    }
-    if (tok == AMPER)
-	YYERROR;
-    for (;;) {
-	if (tok == STRING) {
-	    incmdpos = 0;
-	    addnode(c->args, tokstr);
-	    yylex();
-	} else if (tok == ENVSTRING) {
+	else if (tok == ENVSTRING) {
 	    struct varasg *v = (struct varasg *)make_varnode();
 
 	    v->type = PMFLAG_s;
 	    equalsplit(v->name = tokstr, &v->str);
 	    addnode(c->vars, v);
-	    yylex();
+	    isnull = 0;
 	} else if (tok == ENVARRAY) {
 	    struct varasg *v = (struct varasg *)make_varnode();
 	    int oldcmdpos = incmdpos;
@@ -877,8 +866,19 @@ Cmd c;
 	    if (tok != OUTPAR)
 		YYERROR;
 	    incmdpos = oldcmdpos;
-	    yylex();
 	    addnode(c->vars, v);
+	    isnull = 0;
+	} else
+	    break;
+	yylex();
+    }
+    if (tok == AMPER)
+	YYERROR;
+    for (;;) {
+	if (tok == STRING) {
+	    incmdpos = 0;
+	    addnode(c->args, tokstr);
+	    yylex();
 	} else if (IS_REDIROP(tok)) {
 	    par_redir(c->redir);
 	} else if (tok == INOUTPAR) {
@@ -908,12 +908,6 @@ Cmd c;
     }
     if (isnull && empty(c->redir))
 	return NULL;
-    if (full(c->args)) {
-	if (underscore)
-	    free(underscore);
-	underscore = ztrdup(getdata(lastnode(c->args)));
-	untokenize(underscore);
-    }
     incmdpos = 1;
     return c;
 }
@@ -933,8 +927,8 @@ Cond par_cond()
 	while (tok == SEPER)
 	    yylex();
 	c2 = (Cond) make_cond();
-	c2->left = (vptr) c;
-	c2->right = (vptr) par_cond();
+	c2->left = (void_ptr) c;
+	c2->right = (void_ptr) par_cond();
 	c2->type = COND_OR;
 	return c2;
     }
@@ -956,8 +950,8 @@ Cond par_cond_1()
 	while (tok == SEPER)
 	    yylex();
 	c2 = (Cond) make_cond();
-	c2->left = (vptr) c;
-	c2->right = (vptr) par_cond_1();
+	c2->left = (void_ptr) c;
+	c2->right = (void_ptr) par_cond_1();
 	c2->type = COND_AND;
 	return c2;
     }
@@ -981,7 +975,7 @@ Cond par_cond_2()
 	yylex();
 	c = par_cond_2();
 	c2 = (Cond) make_cond();
-	c2->left = (vptr) c;
+	c2->left = (void_ptr) c;
 	c2->type = COND_NOT;
 	return c2;
     }
@@ -1009,8 +1003,8 @@ Cond par_cond_2()
 	s3 = tokstr;
 	yylex();
 	c = (Cond) make_cond();
-	c->left = (vptr) s1;
-	c->right = (vptr) s3;
+	c->left = (void_ptr) s1;
+	c->right = (void_ptr) s3;
 	c->type = (xtok == INANG) ? COND_STRLT : COND_STRGTR;
 	c->ntype = NT_SET(N_COND, 1, NT_STR, NT_STR, 0, 0);
 	return c;
@@ -1197,7 +1191,7 @@ char *b;
 	zerr("parse error: condition expected: %s", a, 0);
 	return NULL;
     }
-    n->left = (vptr) b;
+    n->left = (void_ptr) b;
     n->type = a[1];
     n->ntype = NT_SET(N_COND, 1, NT_STR, NT_STR, 0, 0);
     return n;
@@ -1237,8 +1231,8 @@ char *c;
 	    zerr("unrecognized condition: %s", b, 0);
     } else
 	zerr("condition expected: %s", b, 0);
-    n->left = (vptr) a;
-    n->right = (vptr) c;
+    n->left = (void_ptr) a;
+    n->right = (void_ptr) c;
     n->ntype = NT_SET(N_COND, 1, NT_STR, NT_STR, 0, 0);
     return n;
 }
