@@ -652,7 +652,7 @@ int always;
 	    zsfree(namdirs[t0].name);
 	    zsfree(namdirs[t0].dir);
 	    for (; t0 < userdirct - 1; t0++)
-		memcpy((vptr) & namdirs[t0], (vptr) & namdirs[t0 + 1],
+		memcpy((void_ptr) & namdirs[t0], (void_ptr) & namdirs[t0 + 1],
 		       sizeof *namdirs);
 	    userdirct--;
 	    finddir(0);
@@ -672,11 +672,11 @@ int always;
 	t1 = namdirs[t0].len;
 	namdirs[t0].len = t2;
 	if (t2 < t1)
-	    qsort((vptr) namdirs, t0 + 1, sizeof *namdirs,
-		  (int (*)DCLPROTO((const void *, const void *)))lencmp);
+	    qsort((void_ptr) namdirs, t0 + 1, sizeof *namdirs,
+		  (int (*) _((const void *, const void *)))lencmp);
 	else
-	    qsort((vptr) & namdirs[t0], userdirct - t0, sizeof *namdirs,
-		  (int (*)DCLPROTO((const void *, const void *)))lencmp);
+	    qsort((void_ptr) & namdirs[t0], userdirct - t0, sizeof *namdirs,
+		  (int (*) _((const void *, const void *)))lencmp);
 
 	finddir(0);		/* Invalidate directory cache */
 	return;
@@ -685,7 +685,7 @@ int always;
 
     if (userdirsz == userdirct) {
 	userdirsz *= 2;
-	namdirs = (Nameddirs) realloc((vptr) namdirs,
+	namdirs = (Nameddirs) realloc((void_ptr) namdirs,
 				      sizeof *namdirs * userdirsz);
 	if (!namdirs)
 	    return;
@@ -694,7 +694,7 @@ int always;
 	if (namdirs[t0].len > t2)
 	    break;
     for (t1 = userdirct; t1 > t0; t1--)
-	memcpy((vptr) & namdirs[t1], (vptr) & namdirs[t1 - 1],
+	memcpy((void_ptr) & namdirs[t1], (void_ptr) & namdirs[t1 - 1],
 	       sizeof *namdirs);
     namdirs[t0].len = t2;
     namdirs[t0].namelen = strlen(s);
@@ -703,8 +703,8 @@ int always;
     namdirs[t0].homedir = ishomedir;
     userdirct++;
     if (t0 && namdirs[t0 - 1].len == t2)
-	qsort((vptr) namdirs, t0 + 1, sizeof *namdirs,
-	      (int (*)DCLPROTO((const void *, const void *)))lencmp);
+	qsort((void_ptr) namdirs, t0 + 1, sizeof *namdirs,
+	      (int (*) _((const void *, const void *)))lencmp);
     finddir(0);
 }
 
@@ -723,8 +723,8 @@ char *t;
 }
 
 int lencmp(first, sec)		/**/
-vptr first;
-vptr sec;
+void_ptr first;
+void_ptr sec;
 {
     int i;
 
@@ -938,7 +938,7 @@ void newcmdnamtab()
 }
 
 void freecmdnam(a)		/**/
-vptr a;
+void_ptr a;
 {
     struct cmdnam *c = (struct cmdnam *)a;
 
@@ -952,7 +952,7 @@ vptr a;
 }
 
 void freecompcond(a)		/**/
-vptr a;
+void_ptr a;
 {
     Compcond cc = (Compcond) a;
     Compcond and, or, c;
@@ -995,7 +995,7 @@ vptr a;
 }
 
 void freecompctl(a)		/**/
-vptr a;
+void_ptr a;
 {
     Compctl cc = (Compctl) a;
 
@@ -1032,7 +1032,7 @@ vptr a;
 }
 
 void freecompctlp(a)		/**/
-vptr a;
+void_ptr a;
 {
     Compctlp ccp = (Compctlp) a;
 
@@ -1040,13 +1040,13 @@ vptr a;
 }
 
 void freestr(a)			/**/
-vptr a;
+void_ptr a;
 {
     zsfree(a);
 }
 
 void freeanode(a)		/**/
-vptr a;
+void_ptr a;
 {
     struct alias *c = (struct alias *)a;
 
@@ -1055,7 +1055,7 @@ vptr a;
 }
 
 void freepm(a)			/**/
-vptr a;
+void_ptr a;
 {
     struct param *pm = (Param) a;
 
@@ -1898,7 +1898,7 @@ static int flagtab[N_COUNT] =
     NT_SET(N_WHILE, 1, NT_NODE, NT_NODE, 0, 0),
     NT_SET(N_VARASG, 1, NT_STR, NT_STR, NT_STR | NT_LIST, 0)};
 
-vptr allocnode(type)		/**/
+void_ptr allocnode(type)		/**/
 int type;
 {
     struct node *n = (struct node *)alloc(sizetab[type]);
@@ -1908,29 +1908,29 @@ int type;
     if (useheap)
 	n->type |= NT_HEAP;
 
-    return (vptr) n;
+    return (void_ptr) n;
 }
 
-vptr dupstruct(a)		/**/
-vptr a;
+void_ptr dupstruct(a)		/**/
+void_ptr a;
 {
     struct node *n = (struct node *)a, *r;
 
     if (!a || ((List) a) == &dummy_list)
-	return (vptr) a;
+	return (void_ptr) a;
 
     if ((n->type & NT_HEAP) && !useheap) {
 	heapalloc();
-	n = (struct node *)dupstruct2((vptr) n);
+	n = (struct node *)dupstruct2((void_ptr) n);
 	permalloc();
 	n = simplifystruct(n);
     }
-    r = (struct node *)dupstruct2((vptr) n);
+    r = (struct node *)dupstruct2((void_ptr) n);
 
     if (!(n->type & NT_HEAP) && useheap)
 	r = expandstruct(r, N_LIST);
 
-    return (vptr) r;
+    return (void_ptr) r;
 }
 
 struct node *simplifystruct(n)	/**/
@@ -2212,43 +2212,43 @@ int exp;
 
 /* duplicate a syntax tree node of given type, argument number */
 
-vptr dupnode(type, a, argnum)	/**/
+void_ptr dupnode(type, a, argnum)	/**/
 int type;
-vptr a;
+void_ptr a;
 int argnum;
 {
     if (!a)
 	return NULL;
     switch (NT_N(type, argnum)) {
     case NT_NODE:
-	return (vptr) dupstruct2(a);
+	return (void_ptr) dupstruct2(a);
     case NT_STR:
-	return (useheap) ? ((vptr) dupstring(a)) :
-	    ((vptr) ztrdup(a));
+	return (useheap) ? ((void_ptr) dupstring(a)) :
+	    ((void_ptr) ztrdup(a));
     case NT_LIST | NT_NODE:
 	if (type & NT_HEAP)
 	    if (useheap)
-		return (vptr) duplist(a, (VFunc) dupstruct2);
+		return (void_ptr) duplist(a, (VFunc) dupstruct2);
 	    else
-		return (vptr) list2arr(a, (VFunc) dupstruct2);
+		return (void_ptr) list2arr(a, (VFunc) dupstruct2);
 	else if (useheap)
-	    return (vptr) arr2list(a, (VFunc) dupstruct2);
+	    return (void_ptr) arr2list(a, (VFunc) dupstruct2);
 	else
-	    return (vptr) duparray(a, (VFunc) dupstruct2);
+	    return (void_ptr) duparray(a, (VFunc) dupstruct2);
     case NT_LIST | NT_STR:
 	if (type & NT_HEAP)
 	    if (useheap)
-		return (vptr) duplist(a, (VFunc) dupstring);
+		return (void_ptr) duplist(a, (VFunc) dupstring);
 	    else
-		return (vptr) list2arr(a, (VFunc) ztrdup);
+		return (void_ptr) list2arr(a, (VFunc) ztrdup);
 	else if (useheap)
-	    return (vptr) arr2list(a, (VFunc) dupstring);
+	    return (void_ptr) arr2list(a, (VFunc) dupstring);
 	else
-	    return (vptr) duparray(a, (VFunc) ztrdup);
+	    return (void_ptr) duparray(a, (VFunc) ztrdup);
     case NT_NODE | NT_ARR:
-	return (vptr) duparray(a, (VFunc) dupstruct2);
+	return (void_ptr) duparray(a, (VFunc) dupstruct2);
     case NT_STR | NT_ARR:
-	return (vptr) duparray(a, (VFunc) (useheap ? dupstring : ztrdup));
+	return (void_ptr) duparray(a, (VFunc) (useheap ? dupstring : ztrdup));
     default:
 	abort();
     }
@@ -2258,7 +2258,7 @@ int argnum;
 
 void freenode(type, a, argnum)	/**/
 int type;
-vptr a;
+void_ptr a;
 int argnum;
 {
     if (!a)
@@ -2291,8 +2291,8 @@ int argnum;
 
 /* duplicate a syntax tree */
 
-vptr *dupstruct2(a)		/**/
-vptr a;
+void_ptr *dupstruct2(a)		/**/
+void_ptr a;
 {
     struct node *n = (struct node *)a, *m;
     int type;
@@ -2345,7 +2345,7 @@ vptr a;
 	    mc->flags = nc->flags;
 	    mc->lineno = nc->lineno;
 	    mc->args = (Lklist) dupnode(type, nc->args, 0);
-	    mc->u.generic = (vptr) dupnode(type, nc->u.generic, 1);
+	    mc->u.generic = (void_ptr) dupnode(type, nc->u.generic, 1);
 	    mc->redir = (Lklist) dupnode(type, nc->redir, 2);
 	    mc->vars = (Lklist) dupnode(type, nc->vars, 3);
 	}
@@ -2367,8 +2367,8 @@ vptr a;
 	    Cond mco = (Cond) m;
 
 	    mco->type = nco->type;
-	    mco->left = (vptr) dupnode(type, nco->left, 0);
-	    mco->right = (vptr) dupnode(type, nco->right, 1);
+	    mco->left = (void_ptr) dupnode(type, nco->left, 0);
+	    mco->right = (void_ptr) dupnode(type, nco->right, 1);
 	}
 	break;
     case N_FOR:
@@ -2422,13 +2422,13 @@ vptr a;
 	}
 	break;
     }
-    return (vptr *) m;
+    return (void_ptr *) m;
 }
 
 /* free a syntax tree */
 
 void freestruct(a)		/**/
-vptr a;
+void_ptr a;
 {
     struct node *n = (struct node *)a;
     int type;
@@ -2703,7 +2703,11 @@ char *oldname;
     char *new = newname, *old;
 
     if (itok(*oldname)) {
+	int ne = noerrs;
+
+	noerrs = 1;
 	singsub(&oldname);
+	noerrs = ne;
 	if (!oldname)
 	    return NULL;
     }
