@@ -118,7 +118,7 @@ char *fmt;
 int prnt;
 int fini;
 {
-    char *p, buf[40], *bf;
+    char *p, buf[40], buf2[80], *bf;
     int i;
     time_t timet;
     struct tm *tm;
@@ -192,7 +192,21 @@ int fini;
 			fm2 = "%m/%d/%y";
 			break;
 		    case 'D':
-			fm2 = "%y-%m-%d";
+			if (fm2[1] == '{') {
+			    char *dd, *ss;
+			    int n = 79;
+
+			    for (ss = fm2 + 2, dd = buf2;
+				 n-- && *ss && *ss != '}'; ++ss, ++dd)
+				*dd = *((*ss == '\\' && ss[1]) ? ++ss : ss);
+			    if (*ss == '}') {
+				*dd = '\0';
+				fmt = ss + 1;
+				fm2 = buf2;
+			    }
+			    else fm2 = "%y-%m-%d";
+			}
+			else fm2 = "%y-%m-%d";
 			break;
 		    }
 		    timet = getlogtime(u, inout);
@@ -339,14 +353,14 @@ void readwtab()
 	{
 	    uptr++;
 	    if (++wtabsz == wtabmax)
-		uptr = (wtab = (struct utmp *)realloc((vptr) wtab, (wtabmax *= 2) *
+		uptr = (wtab = (struct utmp *)realloc((void_ptr) wtab, (wtabmax *= 2) *
 						      sizeof(struct utmp))) + wtabsz;
 	}
     fclose(in);
 
     if (wtabsz)
-	qsort((vptr) wtab, wtabsz, sizeof(struct utmp),
-	           (int (*)DCLPROTO((const void *, const void *)))ucmp);
+	qsort((void_ptr) wtab, wtabsz, sizeof(struct utmp),
+	           (int (*) _((const void *, const void *)))ucmp);
 }
 
 /* check for login/logout events; executed before each prompt
@@ -390,7 +404,7 @@ void dowatch()
 	{
 	    uptr++;
 	    if (++utabsz == utabmax)
-		uptr = (utab = (struct utmp *)realloc((vptr) utab, (utabmax *= 2) *
+		uptr = (utab = (struct utmp *)realloc((void_ptr) utab, (utabmax *= 2) *
 						      sizeof(struct utmp))) + utabsz;
 	}
     fclose(in);
@@ -400,8 +414,8 @@ void dowatch()
 	return;
     }
     if (utabsz)
-	qsort((vptr) utab, utabsz, sizeof(struct utmp),
-	           (int (*)DCLPROTO((const void *, const void *)))ucmp);
+	qsort((void_ptr) utab, utabsz, sizeof(struct utmp),
+	           (int (*) _((const void *, const void *)))ucmp);
 
     wct = wtabsz;
     uct = utabsz;
