@@ -134,7 +134,7 @@ ZLEXTERN char *vichgbuf;
 
 ZLEXTERN int viinsbegin;
 
-typedef void bindfunc DCLPROTO((void));
+typedef void bindfunc _((void));
 typedef bindfunc *F;
 
 struct key {
