@@ -962,7 +962,7 @@ int junc;
 }
 
 void freekey(x)			/**/
-vptr x;
+void_ptr x;
 {
     Key k = (Key) x;
 
