@@ -78,6 +78,9 @@ int usetab()
 {				/**/
     unsigned char *s = line + cs - 1;
 
+    if (!isfirstln)
+	return 0;
+
     for (; s >= line && *s != '\n'; s--)
 	if (*s != '\t' && *s != ' ')
 	    return 0;
@@ -294,19 +297,32 @@ int lst;
 	return;
 
     if (!isfirstln && chline != NULL) {
-	char *p;
+	char *p, *q;
 
-	ol = dupstring((char *) line);
+	ol = dupstring((char *)line);
 	*hptr = '\0';
 	chl = strlen(chline);
+	for (p = chline; *p; p++)
+	    if (*p == '!')
+		chl+=2;
 	sizeline(ll + chl + 1);
-	strcpy((char *) line, chline);
-	strcat((char *) line, ol);
-	for (p = (char *) line; *p; p++)
+	for (p = chline, q = (char *) line; *p; p++) {
+	    if (*p == '!') {
+		*q++ = '\\';
+		*q++ = '\\';
+	    }
+	    *q++ = *p;
+	}
+	*q = '\0';
+	strcat((char *)line, ol);
+	for (p = (char *)line; *p; p++)
 	    if (*p == '\n')
-		*p = ' ';
+		if (p > (char *) line && p[-1] == '\\')
+		    p[-1] = *p = ' ';
+		else
+		    *p = ';';
 	cs += chl;
-	ll = strlen((char *) line);
+	ll = strlen((char *)line);
     }
     else
 	ol = NULL;
@@ -578,7 +594,7 @@ char *get_comp_string()
 	    continue;
 	if (i + 1 == clwsize) {
 	    clwords = (char **)realloc(clwords, (clwsize *= 2) * sizeof(char *));
-	    memset((vptr) (clwords + i), 0, (clwsize / 2) * sizeof(char *));
+	    memset((void_ptr) (clwords + i), 0, (clwsize / 2) * sizeof(char *));
 	}
 	zsfree(clwords[i]);
 	clwords[i] = ztrdup(tokstr);
@@ -825,7 +841,7 @@ int explincmd;
     if (lst == COMP_LIST_EXPAND) {
 	listlist(vl);
 	goto end;
-    } else if (peekfirst(vl) == (vptr) s ||
+    } else if (peekfirst(vl) == (void_ptr) s ||
 	       (!nextnode(firstnode(vl)) && *s == Tilde &&
 		(ss = dupstring(s), filesubstr(&ss, 0)) &&
 		!strcmp(ss, (char *)peekfirst(vl)))) {
@@ -2283,8 +2299,8 @@ char *nam;
 }
 
 int strbpcmp(a, b)		/**/
-const void *a;
-const void *b;
+const void_ptr a;
+const void_ptr b;
 {
     char *aa = *((char **)a), *bb = *((char **)b);
 
@@ -2300,7 +2316,7 @@ const void *b;
 	if (*bb)
 	    bb++;
     }
-    return 0;
+    return (int)(*aa - *bb);
 }
 
 void do_ambiguous()
@@ -2323,8 +2339,8 @@ void do_ambiguous()
 	*ap++ = (char *)getdata(nod);
     *ap = NULL;
 
-    qsort((vptr) amatches, nmatches, sizeof(char *),
-	       (int (*)DCLPROTO((const void *, const void *)))strbpcmp);
+    qsort((void_ptr) amatches, nmatches, sizeof(char *),
+	       (int (*) _((const void *, const void *)))strbpcmp);
 
     for (ap = cp = amatches; *ap; ap++) {
 	*cp++ = *ap;
@@ -2735,8 +2751,8 @@ Lklist l;
 	*ap++ = (char *)getdata(nod);
     *ap = NULL;
 
-    qsort((vptr) amatches, nmatches, sizeof(char *),
-	       (int (*)DCLPROTO((const void *, const void *)))strpcmp);
+    qsort((void_ptr) amatches, nmatches, sizeof(char *),
+	       (int (*) _((const void *, const void *)))strpcmp);
 
     for (ap = cp = amatches; *ap; ap++) {
 	*cp++ = *ap;
