@@ -22,7 +22,7 @@
 #define ZLE
 #include "zsh.h"
 
-SPROTO(void startvichange, (int im));
+static void startvichange _((int im));
 
 static void startvichange(im)
 int im;
@@ -37,7 +37,7 @@ int im;
     viinsbegin = cs;
 }
 
-SPROTO(void startvitext, (int im));
+static void startvitext _((int im));
 
 static void startvitext(im)
 int im;
