@@ -20,7 +20,13 @@
 
 #include "config.h"
 
-#ifdef HAS_UNISTD
+#ifdef __STDC__
+#define _(Args) Args
+#else
+#define _(Args) ()
+#endif
+
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 #ifdef _POSIX_VERSION
@@ -34,13 +40,13 @@
 #include <stdio.h>
 #include <ctype.h>
 
-#ifdef HAS_STRING
+#ifdef HAVE_STRING_H
 #include <string.h>
 #else
 #include <strings.h>
 #endif
 
-#ifdef HAS_MEMORY
+#ifdef HAVE_MEMORY_H
 #include <memory.h>
 #endif
 
@@ -75,7 +81,7 @@
 #include <sys/signal.h>
 #endif
 
-int gethostname();
+int gethostname _((char *,int));
 
 #else				/* not SYSV */
 
@@ -200,7 +206,11 @@ struct timezone {
 #ifdef HAS_TERMIOS
 #define VDISABLEVAL 0
 #define HAS_TIO 1
+#ifdef SCO
+#include <sys/termios.h>
+#else
 #include <termios.h>
+#endif
 #ifdef __sgi
 #ifndef VSWTCH
 #define VSWTCH 7		/* VSWTCH not defined if _POSIX_SOURCE */
@@ -275,26 +285,18 @@ struct timezone {
 #endif
 
 #ifdef __STDC__
-#define DCLPROTO(X) X
-/* prototype template for static functions */
-/* Note that the argument list "a" must have its own ()s around it */
-#define SPROTO(f,a) static f a
 #include <fcntl.h>
 #include <sys/stat.h>
-#ifndef NULL
-#define NULL ((void *)0)
-#endif
 #else				/* K&R C */
-#define DCLPROTO(X) ()
-#define SPROTO(f,a) static f()
 /* SGI cc digs "const" even when K&R */
 #if !(defined(__sgi) && defined(__mips))
 #define const
 #endif
 #include <sys/stat.h>
-#ifndef NULL
-#define NULL 0
 #endif
+
+#ifndef NULL
+#define NULL 0    /* does any system really not define this? */
 #endif
 
 #ifdef HAS_UTMPX
@@ -330,7 +332,7 @@ struct timezone {
 #define EXTERN extern
 #endif
 
-#ifdef HAS_STRING
+#ifdef HAVE_STRING_H
 #define killpg(pgrp,sig) kill(-(pgrp),sig)
 #endif
 
@@ -385,9 +387,7 @@ struct timezone {
 #endif
 #endif
 
-#if 0
-/* The quad_t should be used for BSD 4.4.  There was an ifdef BSD4_4
-   here, but it broke FreeBSD compilation.  */
+#if defined(BSD4_4) && !defined(__FreeBSD__)
 #define RLIM_TYPE quad_t
 #else
 #define RLIM_TYPE long
@@ -405,8 +405,8 @@ typedef int LV;
 /* memory allocation routines - changed with permalloc()/heapalloc() */
 
 /* real things in mem.c */
-extern vptr(*alloc) DCLPROTO((int));
-extern vptr(*ncalloc) DCLPROTO((int));
+extern void_ptr(*alloc) _((int));
+extern void_ptr(*ncalloc) _((int));
 
 /* character tokens */
 
@@ -604,7 +604,7 @@ extern char **environ;
 
 struct lknode {
     struct lknode *next, *last;
-    vptr dat;
+    void_ptr dat;
 };
 struct lklist {
     struct lknode *first, *last;
@@ -637,9 +637,9 @@ typedef struct histent *Histent;
 typedef struct compctlp *Compctlp;
 typedef struct compctl *Compctl;
 typedef struct compcond *Compcond;
-typedef void (*FFunc) DCLPROTO((vptr));
-typedef vptr(*VFunc) DCLPROTO((vptr));
-typedef void (*HFunc) DCLPROTO((char *, char *));
+typedef void (*FFunc) _((void_ptr));
+typedef void_ptr(*VFunc) _((void_ptr));
+typedef void (*HFunc) _((char *, char *));
 
 #define addnode(X,Y) insnode(X,(X)->last,Y)
 #define full(X) ((X)->first != NULL)
@@ -754,7 +754,7 @@ struct cmd {
 	struct whilecmd *whilecmd;
 	Sublist pline;
 	Cond cond;
-	vptr generic;
+	void_ptr generic;
     } u;
     Lklist redir;		/* i/o redirections (struct redir *'s) */
     Lklist vars;		/* param assignments (struct varasg *'s) */
@@ -791,7 +791,7 @@ struct redir {
 struct cond {
     int ntype;
     int type;			/* can be cond_type, or a single letter (-a, -b, ...) */
-    vptr left, right;
+    void_ptr left, right;
 };
 
 #define COND_NOT 0
@@ -894,20 +894,20 @@ struct param {
     }
     u;
     union {			/* functions to call to set value */
-	void (*cfn) DCLPROTO((Param, char *));
-	void (*ifn) DCLPROTO((Param, long));
-	void (*afn) DCLPROTO((Param, char **));
+	void (*cfn) _((Param, char *));
+	void (*ifn) _((Param, long));
+	void (*afn) _((Param, char **));
     }
     sets;
     union {			/* functions to call to get value */
-	char *(*cfn) DCLPROTO((Param));
-	long (*ifn) DCLPROTO((Param));
-	char **(*afn) DCLPROTO((Param));
+	char *(*cfn) _((Param));
+	long (*ifn) _((Param));
+	char **(*afn) _((Param));
     }
     gets;
     int ct;			/* output base or field width */
     int flags;
-    vptr data;			/* used by getfns */
+    void_ptr data;		/* used by getfns */
     char *env;			/* location in environment, if exported */
     char *ename;		/* name of corresponding environment var */
     Param old;			/* old struct for use with local */
@@ -930,6 +930,9 @@ struct param {
 #define PMTYPE (PMFLAG_i|PMFLAG_A)
 #define pmtype(X) ((X)->flags & PMTYPE)
 
+#define MAX_ARRLEN    262144
+
+
 /* variable assignment tree element */
 
 struct varasg {
@@ -978,6 +981,7 @@ struct ttyinfo {
 
 struct job {
     long gleader;		/* process group leader of this job */
+    long other;			/* subjob id or subshell pid */
     int stat;
     char *pwd;			/* current working dir of shell when
 				   this job was spawned */
@@ -993,7 +997,11 @@ struct job {
 #define STAT_DONE 8
 #define STAT_LOCKED 16		/* shell is finished creating this job,	may be
 				   deleted from job table */
+#define STAT_NOPRINT 32		/* job was killed internally, we don't
+				   want to show that */
 #define STAT_INUSE 64		/* this job entry is in use */
+#define STAT_SUPERJOB 128	/* job has a subjob */
+#define STAT_SUBJOB 256		/* job is a subjob */
 
 #define SP_RUNNING -1		/* fake statusp for running jobs */
 
@@ -1276,6 +1284,7 @@ struct option {
 #define LISTAMBIGUOUS '\051'
 #define AUTONAMEDIRS '\052'
 #define CSHJUNKIEPAREN '\053'
+#define NOHISTCLOBBER '\054'
 
 #ifndef GLOBALS
 extern struct option optns[];
@@ -1374,6 +1383,7 @@ struct option optns[] =
     {"listambiguous", LISTAMBIGUOUS},
     {"autonamedirs", AUTONAMEDIRS},
     {"cshjunkieparen", CSHJUNKIEPAREN},
+    {"nohistclobber", NOHISTCLOBBER},
     {NULL, 0}
 };
 
@@ -1746,6 +1756,7 @@ EXTERN char *hackzero;
 /* the hostname */
 
 EXTERN char *hostnam;
+EXTERN char *hostvar;		/* $HOST */
 
 EXTERN char *home;		/* $HOME */
 EXTERN char **pparams;		/* $argv */
@@ -1759,6 +1770,10 @@ EXTERN char *readnullcmd;
 
 EXTERN Lklist locallist;
 
+/* the previous locallist, set when executing builtins */
+
+EXTERN Lklist oldlocallist;
+
 /* what level of localness we are at */
 
 EXTERN int locallevel;
